function activate_phone() {
	'use strict';
	var thephone = null;

	var axfrcall = null;
	var axfrid = null;

	var activecall = null;

	var config = {
		close_delay : 2500,
		video : false,
	}
	var config_valid = true;
	var config_allow_edit = false;

	["user","password","server","realm","close_delay", "video"].forEach(function(k) {
		if (typeof phone_base_config == 'object' && phone_base_config.hasOwnProperty(k)) {
			config[k] = phone_base_config[k];
			$("#phonesip" + k).val(config[k]).prop('disabled', true);
			$("#phonesip" + k + "row").css('display', phone_base_config.hasOwnProperty('display_forced') && phone_base_config["display_forced"] ? "" : "none");
		} else if (!config.hasOwnProperty(k)) {
			config_allow_edit = true;
			var t = $("#phonesip" + k).val();
			if (t != "") {
				config[k] = t;
				localStorage.setItem("org.iewc.phone." + k, t);
			} else {
				config_valid = false;
				if (typeof localStorage != 'undefined') {
					t = localStorage.getItem("org.iewc.phone." + k);
				}
			}
			$("#phonesip" + k).val(t).prop('disabled', false);
		}
	});

	if (!config_valid) {
		$("#phonecredentials").css('display', config_allow_edit ? 'block' : 'none');
		if (!config_allow_edit) {
			$("#phonestatus").text("Invalid configuration.");
		}
		return;
	}

	$("#phonestatus").text("Creating phone UA ...");
	$("#phonecredentials").css('display', 'none');
	if (typeof localStorage != 'undefined')
		$("#targetnum").val(localStorage.getItem("org.iewc.lastdialednumber"));

	var socket = new JsSIP.WebSocketInterface(config.server);
	var configuration = {
		sockets : [ socket ],
		uri : 'sip:' + config.user + "@" + config.realm,
		/* Alternatively we can give ha1 (MD5('user:realm:secret')). Requires knowing the auth realm up front ...*/
		password : config.password,
		register : true,
	};

	thephone = new JsSIP.UA(configuration);

	$("#phonelocalid").text(config.user + "@" + config.realm);

	thephone.on('connecting', function(e) {
		console.log("Attempting connection");
		$("#phonestatus").text("Connecting ...");
	});

	thephone.on('connected', function(e) {
		console.log("Phone is now active.");
		$("#phonestatus").text("Connecting ...");
	});

	thephone.on('disconnected', function(e) {
		$("#phoneactive").css("display", "none");
		$("#phoneinactive").css("display", "block");
		if (config_allow_edit)
			$("#phonecredentials").css('display', 'block');
		console.log("Disconnnected");
	});

	thephone.on('registering', function(e) {
		console.log("Attempting registration.");
		$("#phonestatus").text("Registering ...");
	});
	thephone.on('registered', function(e) {
		console.log("Registration successful");
		$("#phoneactive").css("display", "block");
		$("#phoneinactive").css("display", "none");
		/* in case of error fail we will set alternagively */
		$("#phonestatus").text("Connection failed.");
	});
	thephone.on('registrationFailed', function(e) {
		$("#phonestatus").text("Registration failed: " + e.cause);
		console.log("Phone registration failed: " + e.cause + " (shutting down)");
		thephone.stop();
	});
	thephone.on('newRTCSession', function(rtc_event) {
		var divid;
		var call = rtc_event.session;
		var audio = [];

		function log(x) { return console.log(divid + ": " + x); }

		function statuschange(e)
		{
			var wantedclass;
			if (call.isEnded())
				wantedclass = "failedcall";
			else if (call.isOnHold().local)
				wantedclass = "heldcall";
			else if (call.direction == "incoming" && call.isInProgress())
				wantedclass = "ringingcall";
			else {
				if (activecall && activecall != call)
					activecall.hold();
				activecall = call;

				if (call.isMuted().audio)
					wantedclass = "mutedcall";
				else
					wantedclass = "activecall";
			}
			log("status changed: " + wantedclass);

			["heldcall","mutedcall","activecall","failedcall","ringingcall"].forEach(function(c) {
				if (c == wantedclass)
					$("#" + divid).addClass(c);
				else
					$("#" + divid).removeClass(c);
			});

			if (!call.isEnded()) {
				audio.forEach(function(audioid) {
					var audioobj = document.getElementById(audioid);
					if (audioobj)
						audioobj.muted = call.isOnHold().local;
				});
			}
		}

		function setup_streams() {
			call.connection.onaddstream = function(e) {
				var audioid = divid + "-audio-" + e.stream.id;
				$("#"+divid).append("<audio id=\"" + audioid + "\" autoplay style=\"display:none\"></audio>");
				var audioobj = document.getElementById(audioid);
				audioobj.srcObject = e.stream;
				audio.push(audioid);
			};
			call.connection.onaddtrack = function(e) {
				log('addtrack');
				console.log(e);
			};
		}

		do {
			divid = "call-" + Math.random().toString(36).substring(2,8);
		} while (document.getElementById(divid));

		$("#activecalls").append("<div id=\"" + divid + "\" class=\"callbox\" />");

		if (rtc_event.originator == "local") {
			$("#"+divid).append("<h1>Outgoing call to: " + call.remote_identity.uri.user + "</h1>");
			$("#"+divid).append("<p id=\"status-" + divid + "\">Trying ...</p");
			setup_streams();
		} else {
			$("#"+divid).append("<h1>Incoming call from: " + call.remote_identity.uri.user + "</h1>");
			$("#"+divid).append("<p id=\"status-" + divid + "\">Incoming call ... pending answer</p");
		}
		$("#"+divid).append("<div id=\"controls-" + divid + "\" />");
		$("#controls-"+divid).append("<button id=\"cancel-" + divid + "\">" + (rtc_event.originator == "local" ? "Cancel" : "Reject") + "</button>");
		$("#cancel-" + divid).click(function(e) {
			if (!call.isEnded())
				call.terminate();
		});

		if (rtc_event.originator == "remote") {
			$("#controls-"+divid).append("<button id=\"answer-" + divid + "\">Answer</button>");
			$("#answer-" + divid).click(function(e) {
				console.log(e);
				console.log(call);
				call.answer({ mediaConstraints: { audio:true, video:config.video }});
				setup_streams();
			});
		}

		var callhandlers = {
			progress : function(e) {
				if (!e.response)
					return; /* on incoming call we get this event, presumably generated as part of 100 Trying, but empty response */
				log("progress: " + e.response.reason_phrase);
				$("#status-"+divid).text(e.response.reason_phrase + " ...");
			},
			failed : function(e) {
				var fail_reason = (e.message && typeof e.message.reason_phrase != 'undefined' ? e.message.reason_phrase : e.cause);
				log("failed: " + fail_reason);
				$("#status-"+divid).text("Failed: " + fail_reason + ".");
				$("#"+divid+" button").prop('disabled', true);
				statuschange();
				setTimeout(function() { $("#"+divid).remove(); }, config.close_delay);
			},
			ended : function(e) {
				log("ended");
				$("#status-"+divid).text("Call terminated.");
				if (axfrid == divid)
					$("#xfer-blind").select();
				$("#"+divid+" button").prop('disabled', true);
				statuschange();
				setTimeout(function() { $("#"+divid).remove(); }, config.close_delay);
			},
			confirmed : function(e) {
				var seconds = 0;
				var timer;
				$("#status-"+divid).html("Call answered: <span id=\"timer-" + divid + "\">0:00</span>");
				timer = setInterval(function() {
					/* Note - this will wrap at 24h ... extremely unlikely. */
					if (call.isEnded())
						return clearInterval(timer);
					var date = new Date(null);
					date.setSeconds(++seconds);
					var trim = (seconds < 36000 /* 10h */ ? 1 : 0) + (seconds < 3600 /* 1h */ ? 2 : 0) + (seconds < 600 /* 10m */? 1 : 0);
					var timeString = date.toISOString().substr(11 + trim, 8 - trim);
					$("#timer-"+divid).text(timeString);
				}, 1000);

				log("confirmed");
				$("#cancel-"+divid).text("Disconnect");
				$("#answer-"+divid).remove();
				$("#controls-"+divid).append("&nbsp;<button id=\"hold-" + divid + "\">(Un)hold</button>");
				$("#controls-"+divid).append("&nbsp;<button id=\"mute-" + divid + "\">(Un)mute</button>");
				$("#controls-"+divid).append("<br><button id=\"transfer-" + divid + "\">Transfer</button>");
				$("#controls-"+divid).append("&nbsp;<label><input type=\"radio\" name=\"xfer\" id=\"axfr-" + divid + "\">Select for axfr</label>");
				$("#hold-" + divid).click(function(e) {
					if (call.isOnHold().local) {
						log("unholding");
						call.unhold();
					} else {
						log("holding");
						call.hold();
					}
				});
				$("#mute-" + divid).click(function(e) {
					if (call.isMuted().audio) {
						log("unmuting");
						call.unmute({ audio : true, video : config.video });
					} else {
						log("muting");
						call.mute({ audio : true, video : false });
					}
				});
				$("#transfer-" + divid).click(function() {
					var xfer_target;
					if (axfrcall) {
						xfer_target = axfrcall.remote_identity.uri;
					} else {
						xfer_target = $("#targetnum").val();
					}
					$("#status-"+divid).text("Transfer to " + xfer_target);
					call.refer(xfer_target, {
						replaces : axfrcall,
						eventHandlers : {
							requestSucceeded : function(e) {
								$("#status-"+divid).text("Transfer to " + xfer_target + ": Accepted.");
								log("refer:requestSucceeded");
							},
							requestFailed : function(e) {
								$("#status-"+divid).text("Transfer to " + xfer_target + ": Failed.");
								log("refer:requestFailed");
								console.log(e);
							},
							trying : function(e) {
								log("refer:trying");
								$("#status-" + divid).text("Transfer to " + xfer_target + ": " + e.status_line.reason_phrase + " ...");
							},
							progress : function(e) {
								log("refer:progress");
								console.log(e);
								$("#status-" + divid).text("Transfer to " + xfer_target + ": " + e.status_line.reason_phrase + " ...");
							},
							accepted : function(e) {
								log("refer:accepted");
								console.log(e);
								$("#status-" + divid).text("Transfer to " + xfer_target + ": " + e.status_line.reason_phrase + ".");
								call.terminate(); /* Required else the call lingers around on our end for a bit */
							},
							failed : function(e) {
								log("refer:failed");
								console.log(e);
								$("#status-"+divid).append("Transfer to " + xfer_target + " failed: " + e.status_line.reason_phrase + ".");
							},
						}
					});
				});
				$("#axfr-" + divid).change(function(e) {
					log("Selecting as target for AXFR");
					if (axfrid) {
						$("#transfer-" + axfrid).prop("disabled", false);
					}
					axfrcall = call;
					axfrid = divid;
					$("#transfer-" + divid).prop("disabled", true);
				});
				statuschange();
			},
			hold : statuschange,
			unhold : statuschange,
			muted : statuschange,
			unmuted : statuschange,
		};

		for (var x in callhandlers)
			call.on(x, callhandlers[x]);

		statuschange();

	});

	$("#xfer-blind").change(function(e) {
		if (axfrid) {
			$("#transfer-" + axfrid).prop("disabled", false);
		}
		axfrcall = null;
		axfrid = null;
	});

	$("#addcall").click(function(e) {
		var target = $("#targetnum").val();
		var call;

		if (!target)
			return;

		if (typeof localStorage != 'undefined')
			localStorage.setItem("org.iewc.lastdialednumber", target);

		console.log("requesting new call to " + target);
		call = thephone.call('sip:' + target + "@" + thephone.get('realm'), { mediaConstraints : { audio:true, video:config.video }});
		if (activecall)
			activecall.hold();
		activecall = call;
	});

	if (config.video) {
		navigator.getUserMedia({ audio: false, video: true }, function(stream) {
			var video = document.querySelector("video");
			video.srcObject = stream;
			$("#phonelocalvideo").css("display", "block");
		}, function() {
			alert("Error enabling video.  Force disabling.");
			config.video = false;
		});
	}

	thephone.start();
};

$(document).ready(function() {
	$("#phoneregister").click(function() {
		activate_phone();
	});
	activate_phone();
});
